function bvar_forecast(nlags)
% function bvar_forecast(nlags)
% builds forecats for a bvar model
%
% INPUTS
%    nlags:     number of lags for the bvar
%
% OUTPUTS
%    none
%
% SPECIAL REQUIREMENTS
%    none

% Copyright (C) 2007-2010 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

global options_ oo_ M_

options_ = set_default_option(options_, 'bvar_replic', 2000);
if options_.forecast == 0
    error('bvar_forecast: you must specify "forecast" option')
end
[ny, nx, posterior, prior, forecast_data] = bvar_toolbox(nlags);

sims_no_shock = NaN(options_.forecast, ny, options_.bvar_replic);
sims_with_shocks = NaN(options_.forecast, ny, options_.bvar_replic);

S_inv_upper_chol = chol(inv(posterior.S));

% Option 'lower' of chol() not available in old versions of
% Matlab, so using transpose
XXi_lower_chol = chol(posterior.XXi)';

k = ny*nlags+nx;

% Declaration of the companion matrix
Companion_matrix = diag(ones(ny*(nlags-1),1),-ny);

% Number of explosive VAR models sampled
p = 0;
% Loop counter initialization
d = 0;
while d <= options_.bvar_replic

    Sigma = rand_inverse_wishart(ny, posterior.df, S_inv_upper_chol);

    % Option 'lower' of chol() not available in old versions of
    % Matlab, so using transpose
    Sigma_lower_chol = chol(Sigma)';

    Phi = rand_matrix_normal(k, ny, posterior.PhiHat, Sigma_lower_chol, XXi_lower_chol);
    
    % All the eigenvalues of the companion matrix have to be on or inside the unit circle
    Companion_matrix(1:ny,:) = Phi(1:ny*nlags,:)'; 
    test = (abs(eig(Companion_matrix)));
    if any(test>1.0000000000001)
        p = p+1;
    end
    d = d+1;
    
    % Without shocks
    %lags_data = forecast_data.initval;
    %for t = 1:options_.forecast
    %    X = [ reshape(flipdim(lags_data, 1)', 1, ny*nlags) forecast_data.xdata(t, :) ];
    %    y = X * Phi;
    %    lags_data(1:end-1,:) = lags_data(2:end, :);
    %    lags_data(end,:) = y;
    %    sims_no_shock(t, :, d) = y;
    %end
    
    % With shocks
    lags_data = forecast_data.initval;
    for t = 1:options_.forecast
        X = [ reshape(flipdim(lags_data, 1)', 1, ny*nlags) forecast_data.xdata(t, :) ];
        shock = (Sigma_lower_chol * randn(ny, 1))';
        y = X * Phi + shock;
        lags_data(1:end-1,:) = lags_data(2:end, :);
        lags_data(end,:) = y;
        sims_with_shocks(t, :, d) = y;
    end
end

if p > 0
    disp(' ')
    disp(['Some of the VAR models sampled from the posterior distribution'])
    disp(['were found to be explosive (' num2str(p/options_.bvar_replic) ' percent).'])
    disp(' ')
end


% Store results

%DirectoryName = [ M_.fname '/bvar_forecast' ];
%if ~isdir(DirectoryName)
%    if ~isdir(M_.fname)
%        mkdir(M_.fname);
%    end
%    mkdir(DirectoryName);
%end


%save([ DirectoryName '/simulations.mat'], 'sims_with_shocks');

for i = 1:size(options_.varobs, 2)
    name = options_.varobs(:, i);
    
    %sims = squeeze(sims_no_shock(:,i,:));
    %eval(['oo_.bvarfct.' name '.Mean = mean(sims, 2);']);
    %eval(['oo_.bvarfct.' name '.Median = median(sims, 2);']);
    
    sims = squeeze(sims_with_shocks(:,i,:));
    eval(['oo_.bvarfct.' char(name) '.draws = sims;']);
end

%save([ M_.fname '/forecasts.mat'], 'oo_');
